@@ -11,9 +11,8 @@ use {
             flags_display,
         },
         errors::ProgramError,
-        keys,
+        keys::KEY_FORMAT,
         skin::PanelSkin,
-        stage::*,
         task_sync::Dam,
         verb::*,
     },
@@ -109,12 +108,12 @@ impl Panel {
     /// in the dam
     pub fn do_pending_task(
         &mut self,
-        stage: &Stage,
+        app_state: &mut AppState,
         screen: Screen,
         con: &AppContext,
         dam: &mut Dam,
-    ) {
-        self.mut_state().do_pending_task(stage, screen, con, dam)
+    ) -> Result<(), ProgramError> {
+        self.mut_state().do_pending_task(app_state, screen, con, dam)
     }
 
     pub fn has_pending_task(&self) -> bool {
@@ -255,7 +254,7 @@ impl Panel {
                     _ => false,
                 })
                 .filter_map(|v| v.keys.first())
-                .map(|&k| keys::key_event_desc(k))
+                .map(|&k| KEY_FORMAT.to_string(k))
                 .next()
                 .unwrap_or_else(|| ":start_end_panel".to_string());
 
