@@ -7,7 +7,6 @@ use {
         errors::ProgramError,
         flag::Flag,
         pattern::InputPattern,
-        stage::*,
         task_sync::Dam,
         tree::TreeOptions,
         verb::*,
@@ -147,11 +146,11 @@ impl PanelState for PreviewState {
     /// do the preview filtering if required and not yet done
     fn do_pending_task(
         &mut self,
-        _stage: &Stage,
+        _app_state: &mut AppState,
         _screen: Screen,
         con: &AppContext,
         dam: &mut Dam,
-    ) {
+    ) -> Result<(), ProgramError> {
         if self.pending_pattern.is_some() {
             let old_selection = self
                 .filtered_preview
@@ -170,6 +169,7 @@ impl PanelState for PreviewState {
                 }
             }
         }
+        Ok(())
     }
 
     fn selected_path(&self) -> Option<&Path> {
@@ -177,10 +177,18 @@ impl PanelState for PreviewState {
     }
 
     fn set_selected_path(&mut self, path: PathBuf, con: &AppContext) {
+        let selected_line_number = if self.path == path {
+            self.preview.get_selected_line_number()
+        } else {
+            None
+        };
         if let Some(fp) = &self.filtered_preview {
             self.pending_pattern = fp.pattern();
         };
         self.preview = Preview::new(&path, self.prefered_mode, con);
+        if let Some(number) = selected_line_number {
+            self.preview.try_select_line_number(number);
+        }
         self.path = path;
     }
 
@@ -228,7 +236,6 @@ impl PanelState for PreviewState {
         w: &mut W,
         disc: &DisplayContext,
     ) -> Result<(), ProgramError> {
-        let con = &disc.con;
         let state_area = &disc.state_area;
         if state_area.height < 3 {
             warn!("area too small for preview");
@@ -264,7 +271,7 @@ impl PanelState for PreviewState {
         cw.fill(&styles.preview_title, &SPACE_FILLING)?;
         let preview = self.filtered_preview.as_mut().unwrap_or(&mut self.preview);
         preview.display_info(w, disc.screen, disc.panel_skin, &info_area)?;
-        if let Err(err) = preview.display(w, disc.screen, disc.panel_skin, &self.preview_area, con) {
+        if let Err(err) = preview.display(w, disc, &self.preview_area) {
             warn!("error while displaying file: {:?}", &err);
             if preview.get_mode().is_some() {
                 // means it's not an error already
